"""
@File    :   __init__.py
@Path    :   agrothon_client/
@Time    :   2021/05/28
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.0.7
@Contact :   init Module for Agrothon
"""
__VERSION__ = "1.0.7"
import argparse
import logging
from logging.handlers import RotatingFileHandler
logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(levelname)s - %(message)s [%(filename)s:%(lineno)d]",
    datefmt="%d-%b-%y %H:%M:%S",
    handlers=[
        RotatingFileHandler("Agrothon-Client.txt", maxBytes=50000000, backupCount=10),
        logging.StreamHandler(),
    ],
)
LOGGER = logging.getLogger(__name__)

parser = argparse.ArgumentParser()
parser.add_argument("-y", "--hostname", help="API Server host name", required=True)
parser.add_argument("-a", "--apikey", help="API Key of host", required=True)
parser.add_argument("-u", "--usb", help="USB Port of Arduino", type=str, default="/dev/ttyUSB0")
parser.add_argument("-p1", "--pir1", help="GPIO Pin of PIR1", type=int, default=25)
parser.add_argument("-p2", "--pir2", help="GPIO Pin of PIR2", type=int, default=8)
parser.add_argument("-p3", "--pir3", help="GPIO Pin of PIR3", type=int, default=7)
parser.add_argument("-p4", "--pir4", help="GPIO Pin of PIR4", type=int, default=1)
parser.add_argument("-br", "--baudrate", help="Baud rate of USB Port to read sensor data", type=int, default=9600)
parser.add_argument("-r", "--relay", help="Relay Signalling GPIO pin", type=int, default=12)
args = parser.parse_args()

LOGGER.info("Parsed args")
PIR1_GPIO = args.pir1
PIR2_GPIO = args.pir2
PIR3_GPIO = args.pir3
PIR4_GPIO = args.pir4

RELAY_GPIO = args.relay

USB_BAUD_RATE = args.baudrate
USB_PORT = args.usb
SERVER_API_KEY = args.apikey
HOST = args.hostname
if not HOST.endswith("/"):
    HOST = HOST + "/"


"""
@File    :   __main__.py
@Path    :   agrothon_client/
@Time    :   2021/05/28
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.0.8
@Contact :   ckvbalusu@gmail.com
@Desc    :   Main Module for Agrothon
"""
from .utils import serial_sensor_in, pump_status, motion_intruder_detect
import multiprocessing
import os
import signal
import logging

LOGGER = logging.getLogger(__name__)

# def handler(signalname):
#     def f(signal_received, frame):
#         pass
#     return f

# signal.signal(signal.SIGINT, handler("SIGINT"))
# signal.signal(signal.SIGTERM, handler("SIGTERM"))


def main():
    try:
        pool = multiprocessing.Pool()
        sen_result = pool.apply_async(serial_sensor_in)
        pump_result = pool.apply_async(pump_status)
        intruder_checker = multiprocessing.Process(motion_intruder_detect(), daemon=True)
        intruder_checker.start()
        sen_result.wait()
        pump_result.wait()
    except (KeyboardInterrupt,  SystemExit):
        LOGGER.info("Keyboard interrupt given, exiting ...")
    finally:
        LOGGER.info("Exiting all Programs")
        pool.terminate()
        pool.join()
        pool.close()
        intruder_checker.terminate()
        intruder_checker.join()
        intruder_checker.close()
        os._exit(0)

if __name__ == '__main__':
    main()


"""
@File    :   request_helper.py
@Path    :   agrothon_client/
@Time    :   2021/05/28
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.0.7
@Contact :   ckvbalusu@gmail.com
@Desc    :   API Request helper Module for Agrothon
"""
import requests
import io
from typing import Optional
from agrothon_client import SERVER_API_KEY, HOST
import logging
session = requests.Session()

LOGGER = logging.getLogger(__name__)

def sensor_data_post(json: dict) -> bool:
    base_url = f"{HOST}field/sensor?api_key={SERVER_API_KEY}"
    LOGGER.info("Started posting")
    resp = session.post(base_url, json=json)
    if resp.status_code == 200:
        LOGGER.info(resp.json())
        return True
    else:
        LOGGER.info(resp.status_code)
        return False


def image_poster(image: io.BytesIO) -> bool:
    base_url = f"{HOST}intruder/detect?api_key={SERVER_API_KEY}"
    LOGGER.info("Started posting IMage")
    data = {"image": image}
    resp = session.post(base_url, files=data)
    if resp.status_code == 200:
        return True
    else:
        return False


def pump_status_check() -> Optional[bool]:
    base_url = f"{HOST}pump/?api_key={SERVER_API_KEY}"
    resp = session.get(base_url)
    try:
        if resp.status_code == 200:
            data = resp.json()
            if data["status"]:
                return data["status"]
            elif not data["status"]:
                return False
            else:
                return None
        else:
            return None
    except Exception as e:
        LOGGER.error(e)
        pass


"""
@File    :   utils.py
@Path    :   agrothon_client/
@Time    :   2021/05/28
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.0.8
@Contact :   ckvbalusu@gmail.com
@Desc    :   Utils Module for Agrothon Client
"""

from gpiozero import MotionSensor, OutputDevice
from cv2 import VideoCapture, imencode
from .request_helper import *
import io
import os
import logging
import sys
from serial import Serial
from agrothon_client import (
    USB_PORT,
    USB_BAUD_RATE,
    RELAY_GPIO,
    PIR1_GPIO,
    PIR2_GPIO,
    PIR3_GPIO,
    PIR4_GPIO
    )

serial_in = Serial(USB_PORT, USB_BAUD_RATE)

pump = OutputDevice(RELAY_GPIO, active_high=True, initial_value=True)

pir1 = MotionSensor(PIR1_GPIO)
pir2 = MotionSensor(PIR2_GPIO)
pir3 = MotionSensor(PIR3_GPIO)
pir4 = MotionSensor(PIR4_GPIO)

LOGGER = logging.getLogger(__name__)


def motion_intruder_detect():
    LOGGER.info("Starting Intruder Module")
    while True:
        try:
            if pir1.motion_detected or pir2.motion_detected or pir3.motion_detected or pir4.motion_detected:
                LOGGER.info(f"PIR1 : {pir1.value}, PIR2 : {pir2.value}, PIR3 : {pir3.value}. PIR4 : {pir4.value}")
                LOGGER.info("Launching camera")
                img_cap = VideoCapture(0)
                check, frame = img_cap.read()
                is_success, cv2_img = imencode(".jpg", frame)
                img_cap.release()
                if is_success:
                    data = io.BytesIO(cv2_img)
                    resp = image_poster(data)
                    if resp:
                        LOGGER.info(f"Intruder Detected:{str(resp)}")
                    else:
                        LOGGER.error("maybe nothing found")
        except KeyboardInterrupt:
            LOGGER.info("Exiting, Intruder Module")
            os._exit(0)


def serial_sensor_in():
    """
    This Function just get the Serial lines from Arduino NANO and decode them
    """
    LOGGER.info("Starting Sensor module")
    while True:
        try:
            if serial_in.in_waiting:
                serial_line = serial_in.readline().decode('utf-8').strip()
                list_of_values = serial_line.split(",")
                try:
                    no_of_moist_sens = len(list_of_values)-2
                    moist_list = [] * no_of_moist_sens
                    for i in range(no_of_moist_sens):
                        moist_list.append(float(list_of_values[i]))
                    sensor_dict = {"no_of_sensors": no_of_moist_sens, "moisture": moist_list, "humidity": float(list_of_values[len(list_of_values)-1]), "temperature":float(list_of_values[len(list_of_values)-2])}
                    sensor_data_post(json=sensor_dict)
                except ValueError:
                    LOGGER.error(serial_line)
                    LOGGER.error("DHT Data read failed")
                    pass
        except KeyboardInterrupt:
            LOGGER.info("Exiting Sensor module...")
            os._exit(0)

def pump_status():
    LOGGER.info("Starting Pump status Check")
    while True:
        try:
            resp = pump_status_check()
            if resp:
                pump.off()

                LOGGER.info(f"Pump is ON")
            elif not resp:
                pump.on()
                LOGGER.info(f"Pump is OFF")
            else:
                LOGGER.error("Error Updating PUMP Status")
                pass
        except KeyboardInterrupt:
            pump.on() # Switching off the pump as  program is exiting
            LOGGER.info("Exiting Pump Status check module...")
            os._exit(0)




from setuptools import setup, find_packages
import pathlib
CWD = pathlib.Path(__file__).parent

README = (CWD / "Readme.md").read_text()

setup(
    name='AgroClient',
    version="1.0.8",
    packages=find_packages(),
    long_description=README,
    long_description_content_type="text/markdown",
    url='https://github.com/viswanathbalusu/Agrothon-Client',
    license='AGPL3.0',
    author='viswanathbalusu',
    author_email='ckvbalusu@gmail.com',
    include_package_data=True,
    description='A Client module for Agrothon',
    platforms="any",
    install_requires=[
        "requests==2.25.1",
        "gpiozero==1.5.1",
        "pyserial==3.5",
        "opencv-python==4.5.1.48",
        "RPi.GPIO==0.7.0"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "Operating System :: POSIX :: Linux",
        "Development Status :: 5 - Production/Stable"
    ],
    python_requires=">=3.7",
    entry_points={
        "console_scripts":[
            "AgroClient = agrothon_client.__main__:main"
        ]

    },
)
